import {Injectable} from '@angular/core';
import {HttpClient} from "@angular/common/http";
import {Observable} from "rxjs";
import {environment} from "../../../../environments/environment";
import {AppState} from "../app-state";
import {Router} from "@angular/router";

@Injectable({
  providedIn: 'root'
})
export class ApiService {
  constructor(
    private http: HttpClient,
    private appState: AppState,
    private router: Router
  ) {
  }

  get<T>(url: string, needAuthentication = false, options: any = {}): Observable<T> {
    if (this.checkAuthenticated(needAuthentication, options)) {
      // @ts-ignore
      return this.http.get<string>(environment.apiUrl + url, options)
    }
  }

  post<T>(url: string, body: any, needAuthentication = true, options: any = {}): Observable<T> {
    if (this.checkAuthenticated(needAuthentication, options)) {
      // @ts-ignore
      return this.http.post<T>(environment.apiUrl + url, body, options)
    }
  }

  put<T>(url: string, body: any, needAuthentication = true, options: any = {}): Observable<T> {
    if (this.checkAuthenticated(needAuthentication, options)) {
      // @ts-ignore
      return this.http.put<T>(environment.apiUrl + url, body, options)
    }
  }

  delete<T>(url: string, needAuthentication = true, options: any = {}): Observable<T> {
    if (this.checkAuthenticated(needAuthentication, options)) {
      // @ts-ignore
      return this.http.delete<T>(environment.apiUrl + url, options)
    }
  }

  private checkAuthenticated(needAuthentication: boolean, httpOptions: any): boolean {
    if (needAuthentication) {
      if (!this.appState.isAuthenticated || Date.now() > this.appState.authenticationExpiration) {
        this.navigateToLogin()
        return false
      }
      httpOptions.withCredentials = true
    }
    return true
  }

  private navigateToLogin() {
    this.router.navigate(['/account/login'])
  }
}
